import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;

/**
 * Diese Klasse repraesentiert das Hauptfenster der Tuerme von
 * Hanoi-Praesenzaufgabe.
 * 
 * @author Annabelle Klarl
 * 
 */
public class HanoiFrame extends JFrame implements ActionListener {
    private JButton scheibenbewegungenButton;

    private JTextArea ausgabeBereich;

    /**
     * In diesem Programmstueck wird das Fenster erzeugt
     */
    public HanoiFrame() {
        /* In der Kopfleiste des Fenster steht "HanoiFrame" */
        this.setTitle("HanoiFrame");

        /* Das Fenster ist 500 Pixel breit und 200 Pixel hoch. */
        this.setSize(500, 200);

        /* Hier werden alle Buttons erzeugt. */
        this.scheibenbewegungenButton = new JButton(
                "Anzahl Scheiben eingeben?");

        /* Hier wird der Ausgabe-Bereich erzeugt. */
        this.ausgabeBereich = new JTextArea(60, 100);
        JScrollPane scrollPane = new JScrollPane(this.ausgabeBereich);
        scrollPane.setHorizontalScrollBarPolicy(
            ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);

        /*
         * Der ContentPane ist der Ausschnitt des Fensters, auf dem Widgets d.h.
         * Interaktionselemente (wie eine TextArea oder ein Button) platziert
         * werden koennen.
         */
        Container contentPane = this.getContentPane();
        contentPane.setLayout(new GridLayout(2, 1));
        /* Hier wird der Button platziert. */
        contentPane.add(this.scheibenbewegungenButton);
        /* Hier wird der Ausgabebereich platziert. */
        contentPane.add(scrollPane);

        /*
         * Hier wird der Frame als Listener fuer Knopfdruck-Ereignisse bei
         * jedem der Buttons registriert.
         */
        this.scheibenbewegungenButton.addActionListener(this);

        /*
         * Wird das Fenster geschlossen (d.h. auf X gedrueckt), wird mit Hilfe
         * dieser Programmzeile auch unser Programm ordnungsgemaess beendet.
         */
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        // je nach Button entsprechende Methode ausfuehren
        Object source = e.getSource();
        if (source == this.scheibenbewegungenButton) {
            this.protokolliereScheibenbewegungen();
        }
    }

    /**
     * implementiert die Funktionalitaet des
     * "Protokoll der Scheibenbewegungen"-Buttons
     */
    private void protokolliereScheibenbewegungen() {
        String einlesenAnzahlScheiben = JOptionPane
                .showInputDialog("Wieviele Scheiben: ");
        int anzahlScheiben = Integer.parseInt(einlesenAnzahlScheiben);

        this.ausgabeBereich.setText("");
        this.hanoi(anzahlScheiben, 'A', 'B', 'C');
    }

    /**
     * protokolliert rekursiv die Scheibenbewegungen
     */
    public void hanoi(int scheiben, char quelle, char ablage, char ziel) {
        if (scheiben <= 0) {
            return;
        } else {
            hanoi(scheiben - 1, quelle, ziel, ablage);
            String schritt = "Scheibe von " + quelle + " nach " + ziel + "\n";
            this.ausgabeBereich.append(schritt);
            hanoi(scheiben - 1, ablage, quelle, ziel);
        }
    }
}
