import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTextArea;

/**
 * Diese Klasse repraesentiert das Hauptfenster der Logarithmus-Praesenzaufgabe.
 * 
 * @author Annabelle Klarl
 * 
 */
public class LogarithmusFrame extends JFrame implements ActionListener {
    private JButton iterativButton;
    private JButton rekursivButton;

    private JTextArea ausgabeBereich;

    /**
     * In diesem Programmstueck wird das Fenster erzeugt
     */
    public LogarithmusFrame() {
        /* In der Kopfleiste des Fenster steht "LogarithmusFrame" */
        this.setTitle("LogarithmusFrame");

        /* Das Fenster ist 300 Pixel breit und 200 Pixel hoch. */
        this.setSize(300, 200);

        /* Hier werden alle Buttons erzeugt. */
        this.iterativButton = new JButton("iterative log-Berechnung");
        this.rekursivButton = new JButton("rekursive log-Berechnung");

        /* Hier wird der Ausgabe-Bereich erzeugt. */
        this.ausgabeBereich = new JTextArea(60, 100);
        /*
         * Der ContentPane ist der Ausschnitt des Fensters, auf dem Widgets d.h.
         * Interaktionselemente (wie eine TextArea oder ein Button) platziert
         * werden koennen.
         */
        Container contentPane = this.getContentPane();
        contentPane.setLayout(new GridLayout(3, 1));
        /* Hier werden die Buttons platziert. */
        contentPane.add(this.iterativButton);
        contentPane.add(this.rekursivButton);
        /* Hier wird der Ausgabebereich platziert. */
        contentPane.add(ausgabeBereich);

        /*
         * Hier wird der Frame als Listener fuer Knopfdruck-Ereignisse bei
         * jedem der Buttons registriert.
         */
        this.iterativButton.addActionListener(this);
        this.rekursivButton.addActionListener(this);

        /*
         * Wird das Fenster geschlossen (d.h. auf X gedrueckt), wird mit Hilfe
         * dieser Programmzeile auch unser Programm ordnungsgemaess beendet.
         */
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        // je nach Button entsprechende Methode ausfuehren
        Object source = e.getSource();
        if (source == this.iterativButton) {
            this.berechneLogIterativ();
        } else if (source == this.rekursivButton) {
            this.berechneLogRekursiv();
        }
    }

    /**
     * implementiert die Funktionalitaet des Buttons zur iterative Berechnung
     * des Logarithmus
     */
    private void berechneLogIterativ() {
        String einlesenN = JOptionPane.showInputDialog("Zahl n: ");
        int n = Integer.parseInt(einlesenN);

        int log = logiterativ(n);

        this.ausgabeBereich.setText("Der Logarithmus von n ist " + log);
    }

    /**
     * implementiert die Funktionalitaet des Buttons zur iterative Berechnung
     * des Logarithmus
     */
    private void berechneLogRekursiv() {
        String einlesenN = JOptionPane.showInputDialog("Zahl n: ");
        int n = Integer.parseInt(einlesenN);

        int log = logrek(n);

        this.ausgabeBereich.setText("Der Logarithmus von n ist " + log);
    }

    /**
     * berechnet den Logarithmus von n auf iterative Weise
     */
    private int logiterativ(int n) {
        if (n <= 0) {
            return -1;
        }
        int akk = 0;
        while (n > 1) {
            akk = 1 + akk;
            n = n / 2;
        }
        return akk;
    }

    /**
     * berechnet den Logarithmus von n auf rekursive Weise
     */
    private int logrek(int n) {
        if (n <= 0) {
            return -1;
        } else if (n == 1) {
            return 0;
        } else {
            return 1 + logrek(n / 2);
        }
    }
}
